% Quantile Treatment Effects and Bootstrap Inference under Covariate-Adaptive Randomization 
% Yichong Zhang, Xin Zheng
% 2020/02/14

% [Estimation Functions]

% method = 'ate'    ATE with adjusted standard error
%          'sfe'    QTE with stratified fixed effects 
%          'qte'    QTE with naive standard error
%          'qadj'   QTE with adjusted standard error
%          'qsfe'   QTE with stratified fixed effects
%          'ipw'    ATE IPW with adjusted standard error
%          'qipw'   QTE IPW with adjusted standard error
%          'bnaive' ATE&QTE with weighted bootstrap standard error
%          'cab'    ATE&QTE with covariate-adaptive bootstrap standard error

function result = chong_fn_pure( Y, S, A, method)

% Parameters setting
taulist = 0.1:0.05:0.9;
ntau    = length(taulist);
tauvar  = 0;
Hlist   = 0.5:0.1:1.5;
H       = length(Hlist);
ker     = @(u) normpdf(u);
B       = 1000;
design  = 3;           %index in function AR
mu0     = 0;
mu1     = 0;
pi      = 1/2;
% Common indexes
n  = length(A);
n1 = sum(A);
n0 = n-n1;
Y1 = Y(A==1);
Y0 = Y(A==0);
g  = length(unique(S));


%% **********************************************************************************************
%% ATE adjusted
%% **********************************************************************************************
if strcmp(method, 'ate') == 1

FE   = zeros(n,g);
mus1 = zeros(g,1);
mus0 = zeros(g,1);
n1s  = zeros(g,1);
n0s  = zeros(g,1);

for s = 1:g
    FE(:,s) = (S==s);
    n1s(s)  = sum(A.*(S==s));
    n0s(s)  = sum((1-A).*(S==s));
    mus1(s) = sum(Y.*A.*(S==s))/n1s(s);
    mus0(s) = sum(Y.*(1-A).*(S==s))/n0s(s);
end
    
Y1bar = sum(Y.*A)/n1;
Y0bar = sum(Y.*(1-A))/n0;
sigma21 = sum((Y - Y1bar).^2.*A)/n1;
sigma20 = sum((Y - Y0bar).^2.*(1-A))/n0;
thetahat = Y1bar - Y0bar;
std_T_naive = sqrt(sigma21/n1 + sigma20/n0);
T = (Y1bar - Y0bar)/sqrt(sigma21/n1 + sigma20/n0);

result.beta = thetahat;
result.std  = std_T_naive;
result.t    = T;

%% **********************************************************************************************
%% SFE adjusted
%% **********************************************************************************************

elseif strcmp(method, 'sfe') == 1
    
Y1bar = sum(Y.*A)/n1;
Y0bar = sum(Y.*(1-A))/n0;
FE   = zeros(n,g);
mus1 = zeros(g,1);
mus0 = zeros(g,1);
n1s  = zeros(g,1);
n0s  = zeros(g,1);

for s = 1:g
    FE(:,s) = (S==s);
    n1s(s)  = sum(A.*(S==s));
    n0s(s)  = sum((1-A).*(S==s));
    mus1(s) = sum(Y.*A.*(S==s))/n1s(s);
    mus0(s) = sum(Y.*(1-A).*(S==s))/n0s(s);
end
X = [A,FE];
b = regress(Y,X);
theta_sfe = b(1);
xiYbar = (sum(Y.^2.*A)/n1 - sum( (n1s+n0s)/n.*mus1.^2))/pi ...
    + (sum(Y.^2.*(1-A))/n0 - sum( (n1s+n0s)/n.*mus0.^2))/(1-pi);
xiH = sum((n1s+n0s)/n.*((mus1-Y1bar) - (mus0-Y0bar)).^2 );
xipi = (1-2*pi)^2/(pi^2*(1-pi)^2)*sum(tauvar*(n1s+n0s)/n.*((mu1-Y1bar) - (mu0-Y0bar)).^2 );
std_sfe_adj= sqrt((xiYbar + xiH+xipi)/n);
T_sfe_adj = theta_sfe / sqrt((xiYbar + xiH+xipi)/n);
xiA = sum( tauvar*(n1s+n0s)/n.*( (mus1-Y1bar)/pi + (mus0-Y0bar )/(1-pi)).^2);
std_T_adj = sqrt((xiYbar + xiH+xiA)/n);
T_adj= (Y1bar - Y0bar)/sqrt((xiYbar + xiH+xiA)/n);

result.beta = theta_sfe;
result.std  = [std_T_adj, std_sfe_adj];
result.t    = [T_adj, T_sfe_adj];

%% **********************************************************************************************
%% QTE naive
%% **********************************************************************************************

elseif strcmp(method, 'q') == 1

beta = zeros(ntau,1);
std_q_naive = zeros(ntau,1);
T_q_naive  = zeros(ntau,1);
FE = zeros(n,g);
mus1 = zeros(g,1);
mus0 = zeros(g,1);
n1s = zeros(g,1);
n0s = zeros(g,1);
for t = 1:ntau
    tau = taulist(t);
    q1 = prctile(Y1,100*tau);
    q0 = prctile(Y0,100*tau);
    h1 = 1.06*n1^(-1/5)*std(Y1);
    h0 = 1.06*n0^(-1/5)*std(Y0); 
    beta(t) = prctile(Y1,100*tau) - prctile(Y0,100*tau);
for s = 1:g
    FE(:,s) = (S==s);
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum((tau - (Y < q1)).*A.*(S==s))/n1s(s);
    mus0(s) = sum((tau - (Y < q0)).*(1-A).*(S==s))/n0s(s);
end

f1 = mean(ker((Y1-q1)/h1)/h1);
f0 = mean(ker((Y0-q0)/h0)/h0);

xiYbar = (tau*(1-tau) - sum(mus1.^2.*(n1s+n0s)/n))/pi/f1^2 ...
    + (tau*(1-tau) - sum(mus0.^2.*(n1s+n0s)/n))/(1-pi)/f0^2;
xiH = sum((n1s+n0s)/n.*pi*(1-pi).*(mus1/pi/f1+mus0/(1-pi)/f0).^2);
xipi = sum((n1s+n0s)/n.*(mus1/f1-mus0/f0).^2);
std_q_naive(t) = sqrt(xiYbar + xiH+xipi)/sqrt(n); 
T_q_naive(t) = beta(t)./(sqrt(xiYbar + xiH+xipi)/sqrt(n));
end

result.beta = beta;
result.std  = std_q_naive;
result.t    = T_q_naive;


%% **********************************************************************************************
%% QTE adjusted
%% **********************************************************************************************


elseif strcmp(method, 'qadj') == 1

beta = zeros(1,ntau);
std_q_adj = zeros(ntau,H);
T_q_adj  = zeros(ntau,H);
for t = 1:ntau
tau = taulist(t);
FE = zeros(n,g);
mus1 = zeros(g,1);
mus0 = zeros(g,1);
n1s = zeros(g,1);
n0s = zeros(g,1);
q1= prctile(Y1,100*tau);
q0 = prctile(Y0,100*tau);
h1 = 1.06*n1^(-1/5)*std(Y1);
h0 = 1.06*n0^(-1/5)*std(Y0);
beta(t) = prctile(Y1,100*tau) - prctile(Y0,100*tau);
for s = 1:g
    FE(:,s) = (S==s);
    n1s(s) = sum(A.*(S==s));n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum((tau - (Y < q1)).*A.*(S==s))/n1s(s);
    mus0(s) = sum((tau - (Y < q0)).*(1-A).*(S==s))/n0s(s);
end

for h = 1:H;
h11 = Hlist(h)*h1;
h00 = Hlist(h)*h0;    
f1  = mean(ker((Y1-q1)/h11)/h11);
f0  = mean(ker((Y0-q0)/h00)/h00);

xiYbar = (tau*(1-tau) - sum(mus1.^2.*(n1s+n0s)/n))/pi/f1^2 ...
    +  (tau*(1-tau) - sum(mus0.^2.*(n1s+n0s)/n))/(1-pi)/f0^2;
xiH = sum((n1s+n0s)/n.*tauvar.*(mus1/pi/f1+mus0/(1-pi)/f0).^2);
xipi = sum((n1s+n0s)/n.*(mus1/f1-mus0/f0).^2);
std_q_adj(t,h) = sqrt(xiYbar + xiH+xipi)/sqrt(n); 
T_q_adj(t,h) = beta(t)./(sqrt(xiYbar + xiH+xipi)/sqrt(n));
end
end

result.beta = beta;
result.std  = std_q_adj;
result.t    = T_q_adj;

%% **********************************************************************************************
%% SFE Quantile
%% **********************************************************************************************

elseif strcmp(method, 'qsfe') == 1

std_q_sfe = zeros(ntau,H);
T_q_sfe = zeros(ntau,H);
beta = zeros(1,ntau);
beta_sfe = zeros(1,ntau);
Atilde = zeros(n,1);
n1s = zeros(g,1);
n0s = zeros(g,1);
for s = 1:g
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
end
for i = 1:n
    Atilde(i) = 1*(A(i,1) - n1s(S(i,1))/(n1s(S(i,1))+n0s(S(i,1))));
end
for t = 1:ntau
    tau = taulist(t);
beta(t) = prctile(Y1,100*tau) - prctile(Y0,100*tau);

betatemp = rq_fnm([ones(n,1),Atilde], Y, tau);
beta_sfe(t) = betatemp(2);
tau = taulist(t);
mus1 = zeros(g,1);
mus0 = zeros(g,1);
q1= prctile(Y1,100*tau);
q0 = prctile(Y0,100*tau);
h1 = 1.06*n1^(-1/5)*std(Y1);
h0 = 1.06*n0^(-1/5)*std(Y0);
for s = 1:g
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum((tau - (Y < q1)).*A.*(S==s))/n1s(s);
    mus0(s) = sum((tau - (Y < q0)).*(1-A).*(S==s))/n0s(s);
end

for h = 1:H;
h11 = Hlist(h)*h1;
h00 = Hlist(h)*h0;    
f1  = mean(ker((Y1-q1)/h11)/h11);
f0  = mean(ker((Y0-q0)/h00)/h00);

xiYbar = (tau*(1-tau) - sum(mus1.^2.*(n1s+n0s)/n))/pi/f1^2 ...
    +  (tau*(1-tau) - sum(mus0.^2.*(n1s+n0s)/n))/(1-pi)/f0^2;
xiH = sum((n1s+n0s)/n.*tauvar.*(mus1/pi/f1+mus0/(1-pi)/f0).^2);
xipi = sum((n1s+n0s)/n.*(mus1/f1-mus0/f0).^2);
std_q_sfe(t,h) = sqrt(xiYbar + xiH+xipi)/sqrt(n); 
T_q_sfe(t,h) = beta_sfe(t)./(sqrt(xiYbar + xiH+xipi)/sqrt(n));
end
end

result.beta  = beta_sfe;
result.std  = std_q_sfe;
result.t    = T_q_sfe;

%% **********************************************************************************************
%% IPW adjusted
%% **********************************************************************************************

elseif strcmp(method, 'ipw') == 1
    
Atilde = zeros(n,1);
mus1 = zeros(g,1);
mus0 = zeros(g,1);
n1s = zeros(g,1);
n0s = zeros(g,1);

Y1bar = sum(Y.*A)/n1;
Y0bar = sum(Y.*(1-A))/n0;
for s = 1:g
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum(Y.*A.*(S==s))/n1s(s);
    mus0(s) = sum(Y.*(1-A).*(S==s))/n0s(s);
end
for i = 1:n
    Atilde(i) = 1*(A(i,1) - n1s(S(i,1))/(n1s(S(i,1))+n0s(S(i,1))));
end
P = A-Atilde;

thetahat_ipw = mean(A.*Y./P)-mean((1-A).*Y./(1-P));
xiYbar = (sum(Y.^2.*A)/n1 - sum( (n1s+n0s)/n.*mus1.^2))/pi ...
    + (sum(Y.^2.*(1-A))/n0 - sum( (n1s+n0s)/n.*mus0.^2))/(1-pi);
xiH = sum((n1s+n0s)/n.*((mus1-Y1bar) - (mus0-Y0bar)).^2 );
std_ipw_adj= sqrt((xiYbar + xiH)/n);
T_ipw_adj = thetahat_ipw / sqrt((xiYbar + xiH)/n);

result.beta  = thetahat_ipw;
result.std  = std_ipw_adj;
result.t    = T_ipw_adj;


%% **********************************************************************************************
%% IPW Quantile adjusted
%% **********************************************************************************************

elseif strcmp(method, 'qipw') == 1
    
std_q_ipw = zeros(ntau,H);
T_q_ipw = zeros(ntau,H);
beta = zeros(1,ntau);
beta_sfe = zeros(1,ntau);
qhat = zeros(1,ntau);
Atilde = zeros(n,1);
n1s = zeros(g,1);
n0s = zeros(g,1);
for s = 1:g
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
end
for i = 1:n
    Atilde(i) = 1*(A(i,1) - n1s(S(i,1))/(n1s(S(i,1))+n0s(S(i,1))));
end
P = A-Atilde;

for t = 1:ntau
    tau = taulist(t);
beta(t) = prctile(Y1,100*tau) - prctile(Y0,100*tau);
betatemp = rq_fnm([ones(n,1),Atilde], Y, tau);
beta_sfe(t) = betatemp(2);
qhat(t) = myqr1(Y,A,P,tau*n,ones(n,1))-myqr0(Y,A,P,tau*n,ones(n,1));
tau = taulist(t);
mus1 = zeros(g,1);
mus0 = zeros(g,1);
q1= prctile(Y1,100*tau);
q0 = prctile(Y0,100*tau);
h1 = 1.06*n1^(-1/5)*std(Y1);
h0 = 1.06*n0^(-1/5)*std(Y0);
beta(t) = prctile(Y1,100*tau) - prctile(Y0,100*tau);
for s = 1:g
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
    mus1(s) = sum((tau - (Y < q1)).*A.*(S==s))/n1s(s);
    mus0(s) = sum((tau - (Y < q0)).*(1-A).*(S==s))/n0s(s);
end

for h = 1:H;
h11 = Hlist(h)*h1;
h00 = Hlist(h)*h0;    
f1  = mean(ker((Y1-q1)/h11)/h11);
f0  = mean(ker((Y0-q0)/h00)/h00);
xiYbar = (tau*(1-tau) - sum(mus1.^2.*(n1s+n0s)/n))/pi/f1^2 ...
    +  (tau*(1-tau) - sum(mus0.^2.*(n1s+n0s)/n))/(1-pi)/f0^2;
xiH = sum((n1s+n0s)/n.*tauvar.*(mus1/pi/f1+mus0/(1-pi)/f0).^2);
std_q_ipw(t,h) = sqrt(xiYbar + xiH)/sqrt(n); 
T_q_ipw(t,h) = qhat(t)./(sqrt(xiYbar + xiH)/sqrt(n));
end
end

result.beta  = qhat;
result.std  = std_q_ipw;
result.t    = T_q_ipw;
%% **********************************************************************************************
%% Naive Bootstrap 
%% **********************************************************************************************

elseif strcmp(method, 'bnaive') == 1
    
thetahatstar = zeros(B,1);
thetaSFEstar = thetahatstar; 
thetahatstar_ipw_naive=thetahatstar;
betastar = zeros(B,length(taulist));
betastar_sfe = zeros(B,length(taulist));
qhatstar_naive = betastar_sfe;
Y1bar = sum(Y.*A)/n1;
Y0bar = sum(Y.*(1-A))/n0;
FE   = zeros(n,g);
n1s  = zeros(g,1);
n0s  = zeros(g,1);
beta = zeros(1,ntau);
beta_sfe = zeros(1,ntau);
qhat = zeros(1,ntau);

for s = 1:g
    FE(:,s) = (S==s);
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
end
X = [A,FE];
b = regress(Y,X);
theta_sfe = b(1);

Atilde = zeros(n,1);
for i = 1:n
    Atilde(i) = 1*(A(i,1) - n1s(S(i,1))/(n1s(S(i,1))+n0s(S(i,1))));
end
P = A-Atilde;
for t = 1:ntau
    tau = taulist(t);
    beta(t) = prctile(Y1,100*tau) - prctile(Y0,100*tau);
    betatemp = rq_fnm([ones(n,1),Atilde], Y, tau);
    beta_sfe(t) = betatemp(2);
    qhat(t) = myqr1(Y,A,P,tau*n,ones(n,1))-myqr0(Y,A,P,tau*n,ones(n,1));
end
thetahat_ipw = mean(A.*Y./P)-mean((1-A).*Y./(1-P));



% Bootstrap begins

for b = 1:B
xi = -log(1-rand(n,1));


q1 = zeros(n,g);
q0 = zeros(n,g);
for s = 1:g
       q1(:,s) = (S==s).*A;
       q0(:,s) = (S==s).*(1-A);
end
n1star = sum(A.*xi);
n0star = sum((1-A).*xi);
n1stars = zeros(g,1);
nstars = n1stars;
for s = 1:g
    n1stars(s) = sum(xi.*(S==s).*A);
    nstars(s) = sum(xi.*(S==s)); 
end

Atildestar = zeros(n,1);
for i = 1:n
    Atildestar(i) = A(i) - n1stars(S(i))/nstars(S(i));
end


Y1barstar = sum(Y.*A.*xi)/n1star;
Y0barstar = sum(Y.*(1-A).*xi)/n0star;

thetahatstar(b) = Y1barstar - Y0barstar;


thetaSFEstar(b) = sum(xi.*Y.*Atildestar)/sum(xi.*Atildestar.^2);
betastar_temp = zeros(length(taulist),1);
betastar_sfe_temp = betastar_temp;
qhatstar_temp = betastar_temp;
Pstar = A - Atildestar; 

thetahatstar_ipw_naive(b) = sum(xi.*A.*Y./Pstar)/sum(xi)-sum((1-A).*xi.*Y./(1-Pstar))/sum(xi);
for t = 1:length(taulist)
    tau = taulist(t);
betatemp = rq_fnm([ones(n,1).*xi,A.*xi], Y.*xi, tau);    
betastar_temp(t) = betatemp(2);
betatemp = rq_fnm([ones(n,1).*xi,Atildestar.*xi], Y.*xi, tau);
betastar_sfe_temp(t) = betatemp(2);

qhatstar_temp(t) = myqr1(Y,A,Pstar,tau*n,xi)-myqr0(Y,A,Pstar,tau*n,xi);
end
qhatstar_naive(b,:) = qhatstar_temp;
betastar(b,:) = betastar_temp;
betastar_sfe(b,:) = betastar_sfe_temp;
end
std_t_b_naive = (prctile(thetahatstar,97.5) - prctile(thetahatstar,2.5))/(norminv(0.975) - norminv(0.025));
std_t_sfe_naive = (prctile(thetaSFEstar,97.5) - prctile(thetaSFEstar,2.5))/(norminv(0.975) - norminv(0.025));
std_t_ipw_naive = (prctile(thetahatstar_ipw_naive,97.5) - prctile(thetahatstar_ipw_naive,2.5))/(norminv(0.975) - norminv(0.025));

str_q_b_naive =  (prctile(betastar,97.5,1) - prctile(betastar,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_sfe_naive =  (prctile(betastar_sfe,97.5,1) - prctile(betastar_sfe,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_ipw_naive =  (prctile(qhatstar_naive,97.5,1) - prctile(qhatstar_naive,2.5,1))/(norminv(0.975) - norminv(0.025));



T_t_boot_naive = (Y1bar - Y0bar)/std_t_b_naive;
T_sfe_boot_naive = theta_sfe/std_t_sfe_naive;
T_ipw_boot_naive = thetahat_ipw/std_t_ipw_naive;



T_q_b_naive = beta./str_q_b_naive;
T_q_b_sfe_naive = beta_sfe./str_q_b_sfe_naive;
T_q_b_ipw_naive = qhat./str_q_b_ipw_naive;


result.std  = [std_t_b_naive', std_t_sfe_naive', std_t_ipw_naive'];
result.qstd = [str_q_b_naive', str_q_b_sfe_naive', str_q_b_ipw_naive'];
result.t    = [T_t_boot_naive', T_sfe_boot_naive', T_ipw_boot_naive'];
result.qt   = [T_q_b_naive', T_q_b_sfe_naive', T_q_b_ipw_naive'];

%% **********************************************************************************************
%% Covariate-adaptive Bootstrap 
%% **********************************************************************************************

elseif strcmp(method, 'cab') == 1
FE   = zeros(n,g);
beta = zeros(1,ntau);
beta_sfe = zeros(1,ntau);
qhat = zeros(1,ntau);
Atilde = zeros(n,1);
n1s = zeros(g,1);
n0s = zeros(g,1);

for s = 1:g
    FE(:,s) = (S==s);
    n1s(s) = sum(A.*(S==s));
    n0s(s) = sum((1-A).*(S==s));
end
for i = 1:n
    Atilde(i) = 1*(A(i,1) - n1s(S(i,1))/(n1s(S(i,1))+n0s(S(i,1))));
end
P = A-Atilde;

for t = 1:ntau
    tau = taulist(t);
beta(t) = prctile(Y1,100*tau) - prctile(Y0,100*tau);
betatemp = rq_fnm([ones(n,1),Atilde], Y, tau);
beta_sfe(t) = betatemp(2);
qhat(t) = myqr1(Y,A,P,tau*n,ones(n,1))-myqr0(Y,A,P,tau*n,ones(n,1));
end
thetahat_ipw = mean(A.*Y./P)-mean((1-A).*Y./(1-P));
X = [A,FE];
b = regress(Y,X);
theta_sfe = b(1);
Y1bar = sum(Y.*A)/n1;
Y0bar = sum(Y.*(1-A))/n0;
thetahatstar = zeros(B,1);
thetaSFEstar = thetahatstar;
betastar = zeros(B,length(taulist));
betastar_sfe = zeros(B,length(taulist));
thetahatstar_ipw = thetahatstar;
qhatstar = betastar_sfe;
for b = 1:B
FEstar = zeros(n,g);
Sstar = S(randsample(n,n,true));
Astar = AR(design,Sstar,pi);
q1 = zeros(n,g);
q0 = zeros(n,g);
for s = 1:g
       q1(:,s) = (S==s).*A;
       q0(:,s) = (S==s).*(1-A);
end
Ystar = zeros(n,1);   
n1star = sum(Astar);
n0star = n - n1star;
n1stars = zeros(g,1);
nstars = n1stars;
for s = 1:g
    FEstar(:,s) = (Sstar==s);
    n1stars(s) = sum((Sstar==s).*Astar);
    nstars(s) = sum(Sstar==s); 
end

Atildestar = zeros(n,1);
for i = 1:n
    Atildestar(i) = Astar(i) - n1stars(Sstar(i))/nstars(Sstar(i));
    if Astar(i) == 1
       Ysample = Y(q1(:,Sstar(i))==1,:);
       Ystar(i) = datasample(Ysample,1);
    else
        Ysample = Y(q0(:,Sstar(i))==1,:);
        Ystar(i) = datasample(Ysample,1);
    end 
end

Y1barstar = sum(Ystar.*Astar)/n1star;
Y0barstar = sum(Ystar.*(1-Astar))/n0star;
Y1star = Ystar(Astar==1);
Y0star = Ystar(Astar==0);

thetahatstar(b) = Y1barstar - Y0barstar;

Xstar = [Astar,FEstar];
btemp = regress(Ystar,Xstar);
thetaSFEstar(b) = btemp(1);
betastar_temp = zeros(length(taulist),1);
betastar_sfe_temp = betastar_temp;
qhatstar_temp = betastar_temp;
Pstar = Astar - Atildestar; 

thetahatstar_ipw(b) = mean(Astar.*Ystar./Pstar)-mean((1-Astar).*Ystar./(1-Pstar));


for t = 1:length(taulist)
    tau = taulist(t);
betastar_temp(t) = prctile(Y1star,tau*100) - prctile(Y0star,tau*100);
betatemp = rq_fnm([ones(n,1),Atildestar], Ystar, tau);
betastar_sfe_temp(t) = betatemp(2);
qhatstar_temp(t) = myqr1(Ystar,Astar,Pstar,tau*n,ones(n,1))-myqr0(Ystar,Astar,Pstar,tau*n,ones(n,1));
end

betastar(b,:) = betastar_temp;
betastar_sfe(b,:) = betastar_sfe_temp;
qhatstar(b,:) = qhatstar_temp;
end

std_t_b     = (prctile(thetahatstar,97.5) - prctile(thetahatstar,2.5))/(norminv(0.975) - norminv(0.025));
std_t_b_ipw = (prctile(thetahatstar_ipw,97.5) - prctile(thetahatstar_ipw,2.5))/(norminv(0.975) - norminv(0.025));
std_t_sfe   = (prctile(thetaSFEstar,97.5) - prctile(thetaSFEstar,2.5))/(norminv(0.975) - norminv(0.025));
str_q_b     =  (prctile(betastar,97.5,1) - prctile(betastar,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_sfe =  (prctile(betastar_sfe,97.5,1) - prctile(betastar_sfe,2.5,1))/(norminv(0.975) - norminv(0.025));
str_q_b_ipw =  (prctile(qhatstar,97.5,1) - prctile(qhatstar,2.5,1))/(norminv(0.975) - norminv(0.025));


T_t_b_ipw   = thetahat_ipw/std_t_b_ipw;
T_t_boot    = (Y1bar - Y0bar)/std_t_b;
T_sfe_boot  = theta_sfe/std_t_sfe;
T_q_b       = beta./str_q_b;
T_q_b_sfe   = beta_sfe./str_q_b_sfe;
T_q_b_ipw   = qhat./str_q_b_ipw;

result.std  = [std_t_b', std_t_sfe', std_t_b_ipw'];
result.qstd = [str_q_b', str_q_b_sfe', str_q_b_ipw'];
result.t    = [T_t_b_ipw', T_t_boot', T_sfe_boot'];
result.qt   = [T_q_b', T_q_b_sfe', T_q_b_ipw'];

else
    error('Error input of method name.\n');
end
